/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: add-modify-printer.c,v 1.3 2005/11/10 17:50:58 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <fcntl.h>
#include <errno.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout, gettext("Usage: %s -[a|m] (destination) ...\n"), name);
	exit(1);
}

static void
print_attributes(FILE *fp, papi_attribute_t **attributes, char *delimiter)
{
	char buffer[20480];

	papiAttributeListToString(attributes, delimiter,
		buffer, sizeof (buffer));
	fprintf(fp,"%s%s\n", delimiter, buffer);
	fflush(fp);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_printer_t printer = NULL;
	papi_attribute_t **attributes = NULL;
	char *destination = NULL;
	enum {ADD, MODIFY} type;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "a:m:")) != EOF)
		switch (c) {
		case 'a':
			type = ADD;
			destination = optarg;
		case 'm':
			type = MODIFY;
			destination = optarg;
		}

	if ((destination == NULL) || (optind == ac))
		usage(av[0]);

	for (c = optind; c < ac; c++)
		papiAttributeListFromString(&attributes, PAPI_ATTR_APPEND,
				av[c]);

	status = papiServiceCreate(&svc, NULL, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), destination,
			papiStatusString(status));
	}

	if (type == ADD)
		status = papiPrinterAdd(svc, destination, attributes, &printer);
	else
		status = papiPrinterModify(svc, destination, attributes,
						&printer);
	papiAttributeListFree(attributes);
	if (status == PAPI_OK) {
		printf(gettext("destination: %s\n"), destination);
		attributes = papiPrinterGetAttributeList(printer);
		if (attributes != NULL)
			print_attributes(stdout, attributes, "\nRESULT  ");
		papiPrinterFree(printer);
	} else
		fprintf(stderr, gettext("jobPrinter(Add|Modify): %s: %s\n"),
			destination, papiStatusString(status));

	papiServiceDestroy(svc);

	exit(0);
}
